@@ -384,15 +384,15 @@ inline static void
 copy_record_version (gnutls_session_t session,
                      gnutls_handshake_description_t htype, uint8_t version[2])
 {
-  gnutls_protocol_t lver;
+  const version_entry_st* lver;
 
   if (session->internals.initial_negotiation_completed || htype != GNUTLS_HANDSHAKE_CLIENT_HELLO
       || session->internals.default_record_version[0] == 0)
     {
-      lver = gnutls_protocol_get_version (session);
+      lver = get_version (session);
 
-      version[0] = _gnutls_version_get_major (lver);
-      version[1] = _gnutls_version_get_minor (lver);
+      version[0] = lver->major;
+      version[1] = lver->minor;
     }
   else
     {
@@ -441,7 +441,7 @@ _gnutls_send_tlen_int (gnutls_session_t session, content_type_t type,
   ssize_t cipher_size;
   int retval, ret;
   int send_data_size;
-  uint8_t headers[MAX_RECORD_HEADER_SIZE];
+  uint8_t *headers;
   int header_size;
   const uint8_t *data = _data;
   record_parameters_st *record_params;
@@ -475,21 +475,6 @@ _gnutls_send_tlen_int (gnutls_session_t session, content_type_t type,
         return GNUTLS_E_INVALID_SESSION;
       }
 
-  headers[0] = type;
-
-  /* Use the default record version, if it is
-   * set.
-   */
-  copy_record_version (session, htype, &headers[1]);
-
-  header_size = RECORD_HEADER_SIZE(session);
-  /* Adjust header length and add sequence for DTLS */
-  if (IS_DTLS(session))
-    memcpy(&headers[3], &record_state->sequence_number.i, 8);
-
-  _gnutls_record_log
-    ("REC[%p]: Preparing Packet %s(%d) with length: %d and target length: %d\n", session,
-     _gnutls_packet2str (type), type, (int) data_size, (int) target_length);
 
   if (data_size > MAX_USER_SEND_SIZE(session))
     {
@@ -501,9 +486,6 @@ _gnutls_send_tlen_int (gnutls_session_t session, content_type_t type,
   else
     send_data_size = data_size;
   
-  if (unlikely(send_data_size == 0))
-    return gnutls_assert_val(GNUTLS_E_INTERNAL_ERROR);
-
   /* Only encrypt if we don't have data to send 
    * from the previous run. - probably interrupted.
    */
@@ -519,6 +501,9 @@ _gnutls_send_tlen_int (gnutls_session_t session, content_type_t type,
     }
   else
     {
+      if (unlikely(send_data_size == 0))
+        return gnutls_assert_val(GNUTLS_E_INTERNAL_ERROR);
+
       /* now proceed to packet encryption
        */
       cipher_size = MAX_RECORD_SEND_SIZE(session);
@@ -526,10 +511,27 @@ _gnutls_send_tlen_int (gnutls_session_t session, content_type_t type,
       if (bufel == NULL)
         return gnutls_assert_val(GNUTLS_E_MEMORY_ERROR);
 
+      headers = _mbuffer_get_uhead_ptr(bufel);
+      headers[0] = type;
+      /* Use the default record version, if it is
+       * set. */
+      copy_record_version (session, htype, &headers[1]);
+      header_size = RECORD_HEADER_SIZE(session);
+      /* Adjust header length and add sequence for DTLS */
+      if (IS_DTLS(session))
+        memcpy(&headers[3], &record_state->sequence_number.i, 8);
+
+      _gnutls_record_log
+        ("REC[%p]: Preparing Packet %s(%d) with length: %d and target length: %d\n", session,
+         _gnutls_packet2str (type), type, (int) data_size, (int) target_length);
+
+      _mbuffer_set_udata_size(bufel, cipher_size);
+      _mbuffer_set_uhead_size(bufel, header_size);
+
       ret =
-        _gnutls_encrypt (session, headers, header_size, data,
-                         send_data_size, target_length, _mbuffer_get_udata_ptr (bufel),
-                         cipher_size, type, record_params);
+        _gnutls_encrypt (session, 
+                         data, send_data_size, target_length, 
+                         bufel, type, record_params);
       if (ret <= 0)
         {
           gnutls_assert ();
@@ -539,7 +541,7 @@ _gnutls_send_tlen_int (gnutls_session_t session, content_type_t type,
           return ret;   /* error */
         }
 
-      cipher_size = ret;
+      cipher_size = _mbuffer_get_udata_size(bufel);
       retval = send_data_size;
       session->internals.record_send_buffer_user_size = send_data_size;
 
@@ -552,7 +554,6 @@ _gnutls_send_tlen_int (gnutls_session_t session, content_type_t type,
           return gnutls_assert_val(GNUTLS_E_RECORD_LIMIT_REACHED);
         }
 
-      _mbuffer_set_udata_size (bufel, cipher_size);
       ret = _gnutls_io_write_buffered (session, bufel, mflags);
     }
 
@@ -648,32 +649,73 @@ inline static int
 record_check_version (gnutls_session_t session,
                       gnutls_handshake_description_t htype, uint8_t version[2])
 {
-  if (htype == GNUTLS_HANDSHAKE_CLIENT_HELLO)
+const version_entry_st* vers = get_version (session);
+int diff = 0;
+
+  if (vers->major != version[0] || vers->minor != version[1])
+    diff = 1;
+
+  if (!IS_DTLS(session))
     {
-      /* Reject hello packets with major version higher than 3.
-       */
-      if (!(IS_DTLS(session)) && version[0] > 3)
+      if (htype == GNUTLS_HANDSHAKE_CLIENT_HELLO ||
+          htype == GNUTLS_HANDSHAKE_SERVER_HELLO)
         {
+          if (version[0] != 3)
+            {
+              gnutls_assert ();
+              _gnutls_record_log
+                ("REC[%p]: INVALID VERSION PACKET: (%d) %d.%d\n", session,
+                 htype, version[0], version[1]);
+              return GNUTLS_E_UNSUPPORTED_VERSION_PACKET;
+            }
+        }
+      else if (diff != 0)
+        {
+          /* Reject record packets that have a different version than the
+           * one negotiated. Note that this version is not protected by any
+           * mac. I don't really think that this check serves any purpose.
+           */
           gnutls_assert ();
-          _gnutls_record_log
-            ("REC[%p]: INVALID VERSION PACKET: (%d) %d.%d\n", session,
-             htype, version[0], version[1]);
+          _gnutls_record_log ("REC[%p]: INVALID VERSION PACKET: (%d) %d.%d\n",
+                          session, htype, version[0], version[1]);
+
           return GNUTLS_E_UNSUPPORTED_VERSION_PACKET;
         }
     }
-  else if (htype != GNUTLS_HANDSHAKE_SERVER_HELLO &&
-           gnutls_protocol_get_version (session) !=
-           _gnutls_version_get (version[0], version[1]))
+  else /* DTLS */
     {
-      /* Reject record packets that have a different version than the
-       * one negotiated. Note that this version is not protected by any
-       * mac. I don't really think that this check serves any purpose.
+      /* In DTLS the only information we have here is whether we
+       * expect a handshake message or not.
        */
-      gnutls_assert ();
-      _gnutls_record_log ("REC[%p]: INVALID VERSION PACKET: (%d) %d.%d\n",
-                          session, htype, version[0], version[1]);
+      if (htype == (gnutls_handshake_description_t)-1)
+        {
+          if (diff)
+            {
+              /* Reject record packets that have a different version than the
+               * one negotiated. Note that this version is not protected by any
+               * mac. I don't really think that this check serves any purpose.
+               */
+              gnutls_assert ();
+              _gnutls_record_log ("REC[%p]: INVALID VERSION PACKET: (%d) %d.%d\n",
+                              session, htype, version[0], version[1]);
 
-      return GNUTLS_E_UNSUPPORTED_VERSION_PACKET;
+              return GNUTLS_E_UNSUPPORTED_VERSION_PACKET;
+            }
+        }
+      else if (vers->id > GNUTLS_DTLS1_0 && version[0] > 254)
+        {
+          gnutls_assert ();
+          _gnutls_record_log("REC[%p]: INVALID DTLS VERSION PACKET: (%d) %d.%d\n", session,
+                 htype, version[0], version[1]);
+          return GNUTLS_E_UNSUPPORTED_VERSION_PACKET;
+        }
+      else if (vers->id == GNUTLS_DTLS0_9 && version[0] > 1)
+        {
+          gnutls_assert ();
+          _gnutls_record_log("REC[%p]: INVALID DTLS VERSION PACKET: (%d) %d.%d\n", session,
+                 htype, version[0], version[1]);
+          return GNUTLS_E_UNSUPPORTED_VERSION_PACKET;
+        }
     }
 
   return 0;
@@ -1182,12 +1224,13 @@ begin:
    * is processed and authenticated to avoid someone
    * messing with our windows.
    */
-  if (IS_DTLS(session)) 
+  if (IS_DTLS(session) && session->internals.no_replay_protection == 0)
     {
       ret = _dtls_record_check(record_params, packet_sequence);
       if (ret < 0)
         {
-          _gnutls_audit_log(session, "Discarded duplicate message[%u]: %s\n",
+          _gnutls_audit_log(session, "Discarded duplicate message[%u.%u]: %s\n",
+            (unsigned int)record.sequence.i[0]*256 +(unsigned int)record.sequence.i[1],
             (unsigned int) _gnutls_uint64touint32 (packet_sequence), _gnutls_packet2str (record.type));
           goto sanity_check_error;
         }
@@ -1374,8 +1417,12 @@ _gnutls_recv_int (gnutls_session_t session, content_type_t type,
  * %GNUTLS_E_INTERRUPTED or %GNUTLS_E_AGAIN is returned, you must
  * call this function again, with the same parameters; alternatively
  * you could provide a %NULL pointer for data, and 0 for
- * size. cf. gnutls_record_get_direction(). The errno value EMSGSIZE
- * maps to %GNUTLS_E_LARGE_PACKET.
+ * size. cf. gnutls_record_get_direction(). 
+ *
+ * Note that in DTLS this function will return the %GNUTLS_E_LARGE_PACKET
+ * error code if the send data exceed the data MTU value - as returned
+ * by gnutls_dtls_get_data_mtu(). The errno value EMSGSIZE
+ * also maps to %GNUTLS_E_LARGE_PACKET.
  *
  * Returns: The number of bytes sent, or a negative error code.  The
  *   number of bytes sent might be less than @data_size.  The maximum
