@@ -7,7 +7,7 @@
  *
  * The GnuTLS is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
- * as published by the Free Software Foundation; either version 3 of
+ * as published by the Free Software Foundation; either version 2.1 of
  * the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful, but
@@ -40,6 +40,9 @@
 #include <gnutls_num.h>
 #include <gnutls_extensions.h>
 #include <gnutls_constate.h>
+#include <algorithms.h>
+#include <gnutls_state.h>
+#include <gnutls_db.h>
 
 static int pack_certificate_auth_info (gnutls_session_t,
                                        gnutls_buffer_st * packed_session);
@@ -91,7 +94,11 @@ _gnutls_session_pack (gnutls_session_t session,
 
   _gnutls_buffer_init (&sb);
 
+
   id = gnutls_auth_get_type (session);
+
+  BUFFER_APPEND_NUM(&sb, PACKED_SESSION_MAGIC);
+  BUFFER_APPEND_NUM(&sb, session->security_parameters.timestamp);
   BUFFER_APPEND (&sb, &id, 1);
 
   switch (id)
@@ -102,7 +109,7 @@ _gnutls_session_pack (gnutls_session_t session,
       if (ret < 0)
         {
           gnutls_assert ();
-          return ret;
+          goto fail;
         }
       break;
 #endif
@@ -112,7 +119,7 @@ _gnutls_session_pack (gnutls_session_t session,
       if (ret < 0)
         {
           gnutls_assert ();
-          return ret;
+          goto fail;
         }
       break;
 #endif
@@ -122,7 +129,7 @@ _gnutls_session_pack (gnutls_session_t session,
       if (ret < 0)
         {
           gnutls_assert ();
-          return ret;
+          goto fail;
         }
       break;
 #endif
@@ -131,7 +138,7 @@ _gnutls_session_pack (gnutls_session_t session,
       if (ret < 0)
         {
           gnutls_assert ();
-          return ret;
+          goto fail;
         }
       break;
     default:
@@ -146,20 +153,20 @@ _gnutls_session_pack (gnutls_session_t session,
   if (ret < 0)
     {
       gnutls_assert ();
-      _gnutls_buffer_clear (&sb);
-      return ret;
+      goto fail;
     }
 
   ret = _gnutls_ext_pack (session, &sb);
   if (ret < 0)
     {
       gnutls_assert ();
-      _gnutls_buffer_clear (&sb);
-      return ret;
+      goto fail;
     }
 
-  ret = _gnutls_buffer_to_datum (&sb, packed_session);
+  return _gnutls_buffer_to_datum (&sb, packed_session);
 
+fail:
+  _gnutls_buffer_clear (&sb);
   return ret;
 }
 
@@ -172,6 +179,7 @@ _gnutls_session_unpack (gnutls_session_t session,
 {
   int ret;
   gnutls_buffer_st sb;
+  uint32_t magic;
   uint8_t id;
 
   _gnutls_buffer_init (&sb);
@@ -196,6 +204,14 @@ _gnutls_session_unpack (gnutls_session_t session,
       _gnutls_free_auth_info (session);
     }
 
+  BUFFER_POP_NUM (&sb, magic);
+  if (magic != PACKED_SESSION_MAGIC)
+    {
+      ret = gnutls_assert_val(GNUTLS_E_DB_ERROR);
+      goto error;
+    }
+
+  BUFFER_POP_NUM (&sb, session->internals.resumed_security_parameters.timestamp);
   BUFFER_POP (&sb, &id, 1);
 
   switch (id)
@@ -310,20 +326,16 @@ pack_certificate_auth_info (gnutls_session_t session, gnutls_buffer_st * ps)
     {
 
       BUFFER_APPEND_NUM (ps, info->dh.secret_bits);
-      BUFFER_APPEND_PFX (ps, info->dh.prime.data, info->dh.prime.size);
-      BUFFER_APPEND_PFX (ps, info->dh.generator.data,
+      BUFFER_APPEND_PFX4 (ps, info->dh.prime.data, info->dh.prime.size);
+      BUFFER_APPEND_PFX4 (ps, info->dh.generator.data,
                          info->dh.generator.size);
-      BUFFER_APPEND_PFX (ps, info->dh.public_key.data,
+      BUFFER_APPEND_PFX4 (ps, info->dh.public_key.data,
                          info->dh.public_key.size);
-      BUFFER_APPEND_PFX (ps, info->rsa_export.modulus.data,
-                         info->rsa_export.modulus.size);
-      BUFFER_APPEND_PFX (ps, info->rsa_export.exponent.data,
-                         info->rsa_export.exponent.size);
 
       BUFFER_APPEND_NUM (ps, info->ncerts);
 
       for (i = 0; i < info->ncerts; i++)
-        BUFFER_APPEND_PFX (ps, info->raw_certificate_list[i].data,
+        BUFFER_APPEND_PFX4 (ps, info->raw_certificate_list[i].data,
                            info->raw_certificate_list[i].size);
     }
 
@@ -362,18 +374,13 @@ unpack_certificate_auth_info (gnutls_session_t session, gnutls_buffer_st * ps)
 
   info = _gnutls_get_auth_info (session);
   if (info == NULL)
-    {
-      gnutls_assert ();
-      return GNUTLS_E_INTERNAL_ERROR;
-    }
+    return gnutls_assert_val(GNUTLS_E_INTERNAL_ERROR);
 
   BUFFER_POP_NUM (ps, info->dh.secret_bits);
 
   BUFFER_POP_DATUM (ps, &info->dh.prime);
   BUFFER_POP_DATUM (ps, &info->dh.generator);
   BUFFER_POP_DATUM (ps, &info->dh.public_key);
-  BUFFER_POP_DATUM (ps, &info->rsa_export.modulus);
-  BUFFER_POP_DATUM (ps, &info->rsa_export.exponent);
 
   BUFFER_POP_NUM (ps, info->ncerts);
 
@@ -403,9 +410,6 @@ error:
       _gnutls_free_datum (&info->dh.generator);
       _gnutls_free_datum (&info->dh.public_key);
 
-      _gnutls_free_datum (&info->rsa_export.modulus);
-      _gnutls_free_datum (&info->rsa_export.exponent);
-
       for (j = 0; j < i; j++)
         _gnutls_free_datum (&info->raw_certificate_list[j]);
 
@@ -432,9 +436,13 @@ pack_srp_auth_info (gnutls_session_t session, gnutls_buffer_st * ps)
   int len, ret;
   int size_offset;
   size_t cur_size;
+  const char* username = NULL;
 
   if (info && info->username)
-    len = strlen (info->username) + 1;  /* include the terminating null */
+    {
+      username = info->username;
+      len = strlen (info->username) + 1;  /* include the terminating null */
+    }
   else
     len = 0;
 
@@ -442,7 +450,7 @@ pack_srp_auth_info (gnutls_session_t session, gnutls_buffer_st * ps)
   BUFFER_APPEND_NUM (ps, 0);
   cur_size = ps->length;
 
-  BUFFER_APPEND_PFX (ps, info->username, len);
+  BUFFER_APPEND_PFX4 (ps, username, len);
 
   /* write the real size */
   _gnutls_write_uint32 (ps->length - cur_size, ps->data + size_offset);
@@ -465,7 +473,6 @@ unpack_srp_auth_info (gnutls_session_t session, gnutls_buffer_st * ps)
       return GNUTLS_E_INTERNAL_ERROR;
     }
 
-
   ret =
     _gnutls_auth_info_set (session, GNUTLS_CRD_SRP,
                            sizeof (srp_server_auth_info_st), 1);
@@ -477,12 +484,11 @@ unpack_srp_auth_info (gnutls_session_t session, gnutls_buffer_st * ps)
 
   info = _gnutls_get_auth_info (session);
   if (info == NULL)
-    {
-      gnutls_assert ();
-      return GNUTLS_E_INTERNAL_ERROR;
-    }
+    return gnutls_assert_val(GNUTLS_E_INTERNAL_ERROR);
 
   BUFFER_POP (ps, info->username, username_size);
+  if (username_size == 0)
+    info->username[0] = 0;
 
   ret = 0;
 
@@ -517,15 +523,15 @@ pack_anon_auth_info (gnutls_session_t session, gnutls_buffer_st * ps)
   size_offset = ps->length;
   BUFFER_APPEND_NUM (ps, 0);
   cur_size = ps->length;
-
+  
   if (info)
     {
       BUFFER_APPEND_NUM (ps, info->dh.secret_bits);
-      BUFFER_APPEND_PFX (ps, info->dh.prime.data, info->dh.prime.size);
-      BUFFER_APPEND_PFX (ps, info->dh.generator.data,
-                         info->dh.generator.size);
-      BUFFER_APPEND_PFX (ps, info->dh.public_key.data,
-                         info->dh.public_key.size);
+      BUFFER_APPEND_PFX4 (ps, info->dh.prime.data, info->dh.prime.size);
+      BUFFER_APPEND_PFX4 (ps, info->dh.generator.data,
+                          info->dh.generator.size);
+      BUFFER_APPEND_PFX4 (ps, info->dh.public_key.data,
+                          info->dh.public_key.size);
     }
 
   /* write the real size */
@@ -560,10 +566,7 @@ unpack_anon_auth_info (gnutls_session_t session, gnutls_buffer_st * ps)
 
   info = _gnutls_get_auth_info (session);
   if (info == NULL)
-    {
-      gnutls_assert ();
-      return GNUTLS_E_INTERNAL_ERROR;
-    }
+    return gnutls_assert_val(GNUTLS_E_INTERNAL_ERROR);
 
   BUFFER_POP_NUM (ps, info->dh.secret_bits);
 
@@ -613,13 +616,15 @@ pack_psk_auth_info (gnutls_session_t session, gnutls_buffer_st * ps)
   size_t cur_size;
 
   info = _gnutls_get_auth_info (session);
+  if (info == NULL)
+    return gnutls_assert_val(GNUTLS_E_INTERNAL_ERROR);
 
-  if (info && info->username)
+  if (info->username)
     username_len = strlen (info->username) + 1; /* include the terminating null */
   else
     username_len = 0;
 
-  if (info && info->hint)
+  if (info->hint)
     hint_len = strlen (info->hint) + 1; /* include the terminating null */
   else
     hint_len = 0;
@@ -628,13 +633,13 @@ pack_psk_auth_info (gnutls_session_t session, gnutls_buffer_st * ps)
   BUFFER_APPEND_NUM (ps, 0);
   cur_size = ps->length;
 
-  BUFFER_APPEND_PFX (ps, info->username, username_len);
-  BUFFER_APPEND_PFX (ps, info->hint, hint_len);
+  BUFFER_APPEND_PFX4 (ps, info->username, username_len);
+  BUFFER_APPEND_PFX4 (ps, info->hint, hint_len);
 
   BUFFER_APPEND_NUM (ps, info->dh.secret_bits);
-  BUFFER_APPEND_PFX (ps, info->dh.prime.data, info->dh.prime.size);
-  BUFFER_APPEND_PFX (ps, info->dh.generator.data, info->dh.generator.size);
-  BUFFER_APPEND_PFX (ps, info->dh.public_key.data, info->dh.public_key.size);
+  BUFFER_APPEND_PFX4 (ps, info->dh.prime.data, info->dh.prime.size);
+  BUFFER_APPEND_PFX4 (ps, info->dh.generator.data, info->dh.generator.size);
+  BUFFER_APPEND_PFX4 (ps, info->dh.public_key.data, info->dh.public_key.size);
 
   /* write the real size */
   _gnutls_write_uint32 (ps->length - cur_size, ps->data + size_offset);
@@ -660,10 +665,7 @@ unpack_psk_auth_info (gnutls_session_t session, gnutls_buffer_st * ps)
 
   info = _gnutls_get_auth_info (session);
   if (info == NULL)
-    {
-      gnutls_assert ();
-      return GNUTLS_E_INTERNAL_ERROR;
-    }
+    return gnutls_assert_val(GNUTLS_E_INTERNAL_ERROR);
 
   BUFFER_POP_NUM (ps, username_size);
   if (username_size > sizeof (info->username))
@@ -728,7 +730,7 @@ error:
  *      1 byte the session ID size
  *      x bytes the session ID (32 bytes max)
  *
- *      4 bytes a timestamp
+ *      4 bytes the new record padding flag
  *      4 bytes the ECC curve
  *            -------------------
  *                MAX: 169 bytes
@@ -779,15 +781,18 @@ pack_security_parameters (gnutls_session_t session, gnutls_buffer_st * ps)
   BUFFER_APPEND (ps, session->security_parameters.server_random,
                  GNUTLS_RANDOM_SIZE);
 
-  BUFFER_APPEND_NUM (ps, session->security_parameters.session_id_size);
+  BUFFER_APPEND (ps, &session->security_parameters.session_id_size, 1);
   BUFFER_APPEND (ps, session->security_parameters.session_id,
                  session->security_parameters.session_id_size);
 
   BUFFER_APPEND_NUM (ps, session->security_parameters.max_record_send_size);
   BUFFER_APPEND_NUM (ps, session->security_parameters.max_record_recv_size);
-  BUFFER_APPEND_NUM (ps, session->security_parameters.timestamp);
+  BUFFER_APPEND (ps, &session->security_parameters.new_record_padding, 1);
   BUFFER_APPEND_NUM (ps, session->security_parameters.ecc_curve);
 
+  BUFFER_APPEND_NUM (ps, session->security_parameters.server_sign_algo);
+  BUFFER_APPEND_NUM (ps, session->security_parameters.client_sign_algo);
+
   _gnutls_write_uint32 (ps->length - cur_size, ps->data + size_offset);
 
   return 0;
@@ -798,15 +803,19 @@ unpack_security_parameters (gnutls_session_t session, gnutls_buffer_st * ps)
 {
   size_t pack_size;
   int ret;
-  time_t timestamp = gnutls_time (0);
-
+  time_t timestamp;
+  
   BUFFER_POP_NUM (ps, pack_size);
 
   if (pack_size == 0)
     return GNUTLS_E_INVALID_REQUEST;
 
+  timestamp = session->internals.resumed_security_parameters.timestamp;
   memset (&session->internals.resumed_security_parameters, 0,
           sizeof (session->internals.resumed_security_parameters));
+  session->internals.resumed_security_parameters.timestamp = timestamp;
+ 
+  timestamp = gnutls_time (0);
 
   BUFFER_POP_NUM (ps, session->internals.resumed_security_parameters.entity);
   BUFFER_POP_NUM (ps,
@@ -828,9 +837,8 @@ unpack_security_parameters (gnutls_session_t session, gnutls_buffer_st * ps)
   BUFFER_POP (ps,
               session->internals.resumed_security_parameters.server_random,
               GNUTLS_RANDOM_SIZE);
-  BUFFER_POP_NUM (ps,
-                  session->internals.
-                  resumed_security_parameters.session_id_size);
+  BUFFER_POP (ps, &session->internals.
+                  resumed_security_parameters.session_id_size, 1);
 
   BUFFER_POP (ps, session->internals.resumed_security_parameters.session_id,
               session->internals.resumed_security_parameters.session_id_size);
@@ -841,11 +849,21 @@ unpack_security_parameters (gnutls_session_t session, gnutls_buffer_st * ps)
   BUFFER_POP_NUM (ps,
                   session->internals.
                   resumed_security_parameters.max_record_recv_size);
-  BUFFER_POP_NUM (ps,
-                  session->internals.resumed_security_parameters.timestamp);
+
+  BUFFER_POP (ps, &session->internals.resumed_security_parameters.new_record_padding, 1);
 
   BUFFER_POP_NUM (ps,
                   session->internals.resumed_security_parameters.ecc_curve);
+  BUFFER_POP_NUM (ps,
+                  session->internals.resumed_security_parameters.server_sign_algo);
+  BUFFER_POP_NUM (ps,
+                  session->internals.resumed_security_parameters.client_sign_algo);
+
+  if (session->internals.resumed_security_parameters.max_record_recv_size == 0 ||
+      session->internals.resumed_security_parameters.max_record_send_size == 0)
+    {
+      return gnutls_assert_val(GNUTLS_E_INTERNAL_ERROR);
+    }
 
   if (timestamp - session->internals.resumed_security_parameters.timestamp >
       session->internals.expire_time
@@ -860,3 +878,72 @@ unpack_security_parameters (gnutls_session_t session, gnutls_buffer_st * ps)
 error:
   return ret;
 }
+
+/**
+ * gnutls_session_set_premaster:
+ * @session: is a #gnutls_session_t structure.
+ * @entity: GNUTLS_SERVER or GNUTLS_CLIENT
+ * @version: the TLS protocol version
+ * @kx: the key exchange method
+ * @cipher: the cipher
+ * @mac: the MAC algorithm
+ * @comp: the compression method
+ * @master: the master key to use
+ * @session_id: the session identifier
+ *
+ * This function sets the premaster secret in a session. This is
+ * a function intended for exceptional uses. Do not use this
+ * function unless you are implementing a legacy protocol.
+ * Use gnutls_session_set_data() instead.
+ *
+ * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned, otherwise
+ *   an error code is returned.
+ **/
+int
+gnutls_session_set_premaster (gnutls_session_t session, unsigned int entity,
+                              gnutls_protocol_t version, 
+                              gnutls_kx_algorithm_t kx, 
+                              gnutls_cipher_algorithm_t cipher,
+                              gnutls_mac_algorithm_t mac,
+                              gnutls_compression_method_t comp,
+                              const gnutls_datum_t* master,
+                              const gnutls_datum_t * session_id)
+{
+  int ret;
+
+  memset (&session->internals.resumed_security_parameters, 0,
+          sizeof (session->internals.resumed_security_parameters));
+  
+  session->internals.resumed_security_parameters.entity = entity;
+  session->internals.resumed_security_parameters.kx_algorithm = kx;
+  
+  ret = _gnutls_cipher_suite_get_id(kx, cipher, mac, session->internals.resumed_security_parameters.cipher_suite);
+  if (ret < 0)
+    return gnutls_assert_val(ret);
+
+  session->internals.resumed_security_parameters.compression_method = comp;
+  session->internals.resumed_security_parameters.cert_type = DEFAULT_CERT_TYPE;
+  session->internals.resumed_security_parameters.version = version;
+
+  if (master->size != GNUTLS_MASTER_SIZE)
+    return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
+
+  memcpy(session->internals.resumed_security_parameters.master_secret, master->data, master->size);
+
+  if (session_id->size > GNUTLS_MAX_SESSION_ID)
+    return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
+
+  session->internals.resumed_security_parameters.session_id_size = session_id->size;
+  memcpy(session->internals.resumed_security_parameters.session_id, session_id->data, session_id->size);
+
+  session->internals.resumed_security_parameters.max_record_send_size = 
+  session->internals.resumed_security_parameters.max_record_recv_size = DEFAULT_MAX_RECORD_SIZE;
+
+  session->internals.resumed_security_parameters.timestamp = gnutls_time(0);
+
+  session->internals.resumed_security_parameters.ecc_curve = GNUTLS_ECC_CURVE_INVALID;
+
+  session->internals.premaster_set = 1;
+
+  return 0;
+}
