@@ -117,7 +117,7 @@ void start_benchmark(struct benchmark_st * st)
 }
 
 /* returns the elapsed time */
-double stop_benchmark(struct benchmark_st * st, const char* metric)
+double stop_benchmark(struct benchmark_st * st, const char* metric, int quiet)
 {
   double secs;
   unsigned long lsecs;
@@ -144,14 +144,16 @@ double stop_benchmark(struct benchmark_st * st, const char* metric)
   if (metric == NULL)
     { /* assume bytes/sec */
       value2human (st->size, secs, &ddata, &dspeed, imetric);
-      printf ("  Processed %.2f %s in %.2f secs: ", ddata, imetric, secs);
+      if (quiet == 0) 
+        printf ("  Processed %.2f %s in %.2f secs: ", ddata, imetric, secs);
       printf ("%.2f %s/sec\n", dspeed, imetric);
     }
   else
     {
       ddata = (double) st->size;
       dspeed = ddata / secs;
-      printf ("  Processed %.2f %s in %.2f secs: ", ddata, metric, secs);
+      if (quiet == 0) 
+        printf ("  Processed %.2f %s in %.2f secs: ", ddata, metric, secs);
       printf ("%.2f %s/sec\n", dspeed, metric);
     }
 
