@@ -139,7 +139,7 @@ cipher_to_flags (const char *cipher)
 {
   if (cipher == NULL)
     {
-      return GNUTLS_PKCS_USE_PBES2_AES_128;
+      return GNUTLS_PKCS_USE_PKCS12_ARCFOUR;
     }
   else if (strcasecmp (cipher, "3des") == 0)
     {
@@ -240,6 +240,7 @@ generate_certificate (gnutls_privkey_t * ret_key,
   int ret;
   int client;
   int days, result, ca_status = 0, is_ike = 0, path_len;
+  time_t secs, now;
   int vers;
   unsigned int usage = 0, server;
   gnutls_x509_crq_t crq;        /* request */
@@ -288,7 +289,7 @@ generate_certificate (gnutls_privkey_t * ret_key,
           get_dc_set (TYPE_CRT, crt);
 
           get_oid_crt_set (crt);
-          get_key_purpose_set (crt);
+          get_key_purpose_set (TYPE_CRT, crt);
 
           if (!batch)
             fprintf (stderr,
@@ -329,11 +330,17 @@ generate_certificate (gnutls_privkey_t * ret_key,
 
   gnutls_x509_crt_set_activation_time (crt, time (NULL));
 
-  days = get_days ();
+  now = time(NULL);
+  
+  do 
+    {
+      days = get_days ();
+      secs = days * 24 * 60 * 60 + now;
+    }
+  while (secs < now || (unsigned)(secs-now)/(24*60*60) != (unsigned)days);
 
   result =
-    gnutls_x509_crt_set_expiration_time (crt,
-                                         time (NULL) + ((time_t) days) * 24 * 60 * 60);
+    gnutls_x509_crt_set_expiration_time (crt, secs);
   if (result < 0)
     error (EXIT_FAILURE, 0, "set_expiration: %s", gnutls_strerror (result));
 
@@ -1456,7 +1463,7 @@ print_certificate_info (gnutls_x509_crt_t crt, FILE * out, unsigned int all)
     }
 
   if (out == stderr && batch == 0)      /* interactive */
-    if (read_yesno ("Is the above information ok? (y/N): ") == 0)
+    if (read_yesno ("Is the above information ok? (y/N): ", 0) == 0)
       {
         exit (1);
       }
@@ -1909,6 +1916,8 @@ generate_request (common_info_st * cinfo)
           if (ret < 0)
             error (EXIT_FAILURE, 0, "key_kp: %s", gnutls_strerror (ret));
         }
+
+      get_key_purpose_set (TYPE_CRQ, crq);
     }
 
   ret = gnutls_x509_crq_set_pubkey (crq, pubkey);
